import { isString, isArray, isPlainObject, clone } from '@ntks/toolbox';
import { getDataType, isDataTypeValid, isDataValueValid } from './data-type';
import { runExpression } from './context/view/base';
const propCheckerMap = new Map();
function registerInputPropCheckers(dataType, checkers) {
    propCheckerMap.set(dataType, checkers);
}
function resolveInput(refOrDescriptor) {
    return isString(refOrDescriptor)
        ? { name: refOrDescriptor }
        : refOrDescriptor;
}
function getDefaultValue({ dataType }) {
    if (!dataType) {
        return '';
    }
    return isDataTypeValid(dataType) ? getDataType(dataType).defaultValueGetter() : '';
}
function createValueValidator(valueCheckers) {
    return (value, ctx) => {
        const checkers = [].concat(valueCheckers);
        let result = { success: true };
        while (checkers.length > 0) {
            const validate = checkers.shift();
            result = validate(value, ctx);
            if (!result.type) {
                result.type = 'custom';
            }
            if (!result.success) {
                break;
            }
        }
        return result;
    };
}
function createInputValidator(input) {
    const checkers = [];
    const { dataType } = input;
    if (input.required) {
        checkers.push((value, ctx) => {
            const result = { type: 'required', success: true };
            const dynamicRequired = isString(input.required);
            let computedResult = false;
            if (dynamicRequired) {
                computedResult = !!runExpression(ctx, input.required);
            }
            if (!dynamicRequired || computedResult) {
                if (value == null) {
                    result.success = false;
                }
                else if (isString(value)) {
                    result.success = value !== '';
                }
                else if (isArray(value)) {
                    result.success = value.length > 0;
                }
                else if (isPlainObject(value)) {
                    result.success = Object.keys(value).length > 0;
                }
                if (!result.success) {
                    result.message = `请填入 '${input.label || input.name}' 的值`;
                }
            }
            return result;
        });
    }
    if (dataType) {
        checkers.push(value => {
            const valid = isDataValueValid(dataType, value);
            return valid
                ? { type: 'data-type', success: true }
                : {
                    type: 'data-type',
                    success: false,
                    message: `'${input.label || input.name}' 的值 ${value} 与数据类型 '${dataType}' 不符`,
                };
        });
        const propCheckers = propCheckerMap.get(dataType);
        if (propCheckers) {
            const checkerMap = {};
            propCheckers.forEach(({ name, validator }) => {
                checkerMap[name] = validator;
                checkers.push(value => validator(value, input[name], clone(input)));
            });
        }
    }
    return {
        addChecker: checker => checkers.push(checker),
        validate: createValueValidator(checkers),
    };
}
export { registerInputPropCheckers, resolveInput, getDefaultValue, createInputValidator };
